;------------------------------------------------------------------------------
; delay_20.asm
;
; This module provides delay routines for a 20 MHz clock.
;
; @author Cam Thompson, Micromega Corporation
; Copyright (c) Micromega Corporation 2004. All rights reserved.
;
; @version	V1.0 October 15, 2004
; changes:
;	V1.0 - October 15, 2004
;	  - original code
;------------------------------------------------------------------------------

	list      p=16f628				;list directive to define processor
	#include <p16f628.inc>			;processor specific variable definitions
	
;-------------------- variable definitions ------------------------------------
		udata
delayCount1		res	1				;delay count1
delayCount2		res	1				;delay count2

PROG	code

;-------------------- delay_ms ------------------------------------------------
; Delays from 1 to 256 milliseconds (assumes 20 MHz clock)
; Note: The delay will have an additional 1 usec of overhead delay.
;	input:	W register	- number of milliseconds to delay (0 = 256 msec)
;	output:	none
;------------------------------------------------------------------------------

	global		delay_ms
delay_ms
	movwf		delayCount2			;save number of msecs to delay
	
delay_ms2
	movlw		.250				;delay 1 msec per loop
	call		delay
	movlw		.250
	call		delay
	movlw		.250
	call		delay
	movlw		.249
	call		delay
	nop
	nop
	decfsz		delayCount2, f
	goto		delay_ms2
	return

;-------------------- delay ---------------------------------------------------
; Delay for n time units, where n can be 2 to 255.
; The time unit depends on the clock frequency.  The value of the time unit in
; microseconds is equal to 20 / clock frequency.
;
; For a 20 MHz clock the time unit is 1 usec.
; Delays less than 2 usec should be implemented with NOP instructions.
;
;	input:	W register	- number of time unts to delay
;	output:	none
;------------------------------------------------------------------------------

	global		delay
delay
	movwf		delayCount1			;save number of delay units
	decf		delayCount1, f		;adjust for overhead
	decf		delayCount1, f
	btfss		STATUS, Z			;check for minimum delay
	goto		delay2				;no, then loop
	return							;yes then exit
		
delay2
	nop								;delay one time unit per loop
	nop
	decfsz		delayCount1, f
	goto		delay2
	return

	end

